@@ -1,4 +1,4 @@
-%# Copyright (C) 2006-2009, Thomas Treichl <treichl@users.sourceforge.net>
+%# Copyright (C) 2006-2011, Thomas Treichl <treichl@users.sourceforge.net>
 %# OdePkg - A package for solving ordinary differential equations and more
 %#
 %# This program is free software; you can redistribute it and/or modify
@@ -187,7 +187,7 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
   %# Implementation of the option MaxStep has been finished. This option
   %# can be set by the user to another value than default value.
   if (isempty (vodeoptions.MaxStep) && ~vstepsizefixed)
-    vodeoptions.MaxStep = (vslot(1,2) - vslot(1,1)) / 10;
+    vodeoptions.MaxStep = abs (vslot(1,2) - vslot(1,1)) / 10;
     warning ('OdePkg:InvalidArgument', ...
       'Option "MaxStep" not set, new value %f is used', vodeoptions.MaxStep);
   end
@@ -267,10 +267,17 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
   vtimestamp  = vslot(1,1);           %# timestamp = start time
   vtimelength = length (vslot);       %# length needed if fixed steps
   vtimestop   = vslot(1,vtimelength); %# stop time = last value
-  vdirection  = sign (vtimestop);     %# Flag for direction to solve
+  %# 20110611, reported by Nils Strunk
+  %# Make it possible to solve equations from negativ to zero, 
+  %# eg. vres = ode54 (@(t,y) y, [-2 0], 2);
+  vdirection  = sign (vtimestop - vtimestamp); %# Direction flag
 
   if (~vstepsizefixed)
-    vstepsize = vodeoptions.InitialStep;
+    if (sign (vodeoptions.InitialStep) == vdirection)
+      vstepsize = vodeoptions.InitialStep;
+    else %# Fix wrong direction of InitialStep.
+      vstepsize = - vodeoptions.InitialStep;
+    end
     vminstepsize = (vtimestop - vtimestamp) / (1/eps);
   else %# If step size is given then use the fixed time steps
     vstepsize = vslot(1,2) - vslot(1,1);
@@ -310,18 +317,40 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
   %# The solver main loop - stop if the endpoint has been reached
   vcntloop = 2; vcntcycles = 1; vu = vinit; vk = vu.' * zeros(1,7);
   vcntiter = 0; vunhandledtermination = true; vcntsave = 2;
+
+  %# FSAL optimizations: sent by Bruce Minaker 20110326, find the slope k1
+  %# (k1 is copied from last k7, so set k7) for first time step only
+  if (vhavemasshandle)   %# Handle only the dynamic mass matrix,
+    if (vmassdependence) %# constant mass matrices have already
+      vmass = feval ...  %# been set before (if any)
+        (vodeoptions.Mass, vtimestamp, vinit.', vfunarguments{:});
+    else                 %# if (vmassdependence == false)
+      vmass = feval ...  %# then we only have the time argument
+        (vodeoptions.Mass, vtimestamp, vfunarguments{:});
+    end
+    vk(:,7) = vmass \ feval ...
+      (vfun, vtimestamp, vinit.', vfunarguments{:});
+  else
+    vk(:,7) = feval ...
+      (vfun, vtimestamp, vinit.', vfunarguments{:});
+  end
+
   while ((vdirection * (vtimestamp) < vdirection * (vtimestop)) && ...
          (vdirection * (vstepsize) >= vdirection * (vminstepsize)))
 
     %# Hit the endpoint of the time slot exactely
-    if ((vtimestamp + vstepsize) > vdirection * vtimestop)
-%# if (((vtimestamp + vstepsize) > vtimestop) || ...
-%#   (abs(vtimestamp + vstepsize - vtimestop) < eps))
-      vstepsize = vtimestop - vdirection * vtimestamp;
+    if (vdirection * (vtimestamp + vstepsize) > vdirection * vtimestop)
+      %# vstepsize = vtimestop - vdirection * vtimestamp;
+      %# 20110611, reported by Nils Strunk
+      %# The endpoint of the time slot must be hit exactly,
+      %# eg. vsol = ode54 (@(t,y) y, [0 -1], 1); 
+      vstepsize = vdirection * abs (abs (vtimestop) - abs (vtimestamp));
     end
 
-    %# Estimate the seven results when using this solver
-    for j = 1:7
+    %# Estimate the seven results when using this solver (FSAL)
+    %# skip the first result as we already know it from last time step
+    vk(:,1)=vk(:,7);
+    for j = 2:7              %# Start at two instead of one (FSAL)
       vthetime  = vtimestamp + vc(j,1) * vstepsize;
       vtheinput = vu.' + vstepsize * vk(:,1:j-1) * va(j,1:j-1).';
       if (vhavemasshandle)   %# Handle only the dynamic mass matrix,
@@ -341,19 +370,24 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
     end
 
     %# Compute the 4th and the 5th order estimation
-    y4 = vu.' + vstepsize * (vk * vb4);
-    y5 = vu.' + vstepsize * (vk * vb5);
+    y4 = vu.' + vstepsize * (vk * vb4); 
+    y5 = vtheinput;
+    %# y5 = vu.' + vstepsize * (vk * vb5); vb5 is the same as va(6,:), 
+    %# means that we already know y5 from the first six vk's (FSAL)
+
     if (vhavenonnegative)
       vu(vodeoptions.NonNegative) = abs (vu(vodeoptions.NonNegative));
       y4(vodeoptions.NonNegative) = abs (y4(vodeoptions.NonNegative));
       y5(vodeoptions.NonNegative) = abs (y5(vodeoptions.NonNegative));
     end
-    vSaveVUForRefine = vu;
+    if (vhaveoutputfunction && vhaverefine) 
+      vSaveVUForRefine = vu;
+    end
 
     %# Calculate the absolute local truncation error and the acceptable error
     if (~vstepsizefixed)
       if (~vnormcontrol)
-        vdelta =  abs (y5 - y4);
+        vdelta = abs (y5 - y4);
         vtau = max (vodeoptions.RelTol * abs (vu.'), vodeoptions.AbsTol);
       else
         vdelta = norm (y5 - y4, Inf);
@@ -370,8 +404,13 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
       vu = y5.'; %# MC2001: the higher order estimation as "local extrapolation"
       %# Save the solution every vodeoptions.OutputSave steps             
       if (mod (vcntloop-1,vodeoptions.OutputSave) == 0)             
-        vretvaltime(vcntsave,:) = vtimestamp;
-        vretvalresult(vcntsave,:) = vu;
+        if (vhaveoutputselection)
+          vretvaltime(vcntsave,:) = vtimestamp;
+          vretvalresult(vcntsave,:) = vu(vodeoptions.OutputSel);
+        else
+          vretvaltime(vcntsave,:) = vtimestamp;
+          vretvalresult(vcntsave,:) = vu;
+        end
         vcntsave = vcntsave + 1;    
       end     
       vcntloop = vcntloop + 1; vcntiter = 0;
@@ -417,6 +456,12 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
           vunhandledtermination = false; break;
         end
       end
+    else
+      vk(:,7) = vk(:,1);       
+      %# If we're here, then we've overwritten the k7 that we
+      %# need to copy to k1 to redo the step Since we copy k7
+      %# into k1, we'll put the k1 back in k7 for now, until it's
+      %# copied back again (FSAL)
     end %# If the error is acceptable ...
 
     %# Update the step size for the next integration step
@@ -434,7 +479,7 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
         vstepsize = min (vodeoptions.MaxStep, ...
            min (0.8 * vstepsize * (vtau ./ vdelta) .^ vpow));
       else
-        vstepsize = max (vodeoptions.MaxStep, ...
+        vstepsize = max (- vodeoptions.MaxStep, ...
           max (0.8 * vstepsize * (vtau ./ vdelta) .^ vpow));
       end
 
@@ -503,15 +548,15 @@ function [varargout] = ode54 (vfun, vslot, vinit, varargin)
     vhavestats = true;
     vnsteps    = vcntloop-2;                    %# vcntloop from 2..end
     vnfailed   = (vcntcycles-1)-(vcntloop-2)+1; %# vcntcycl from 1..end
-    vnfevals   = 7*(vcntcycles-1);              %# number of ode evaluations
+    vnfevals   = 6*(vcntcycles-1)+1;            %# 7 stages & one first step (FSAL)
     vndecomps  = 0;                             %# number of LU decompositions
     vnpds      = 0;                             %# number of partial derivatives
     vnlinsols  = 0;                             %# no. of solutions of linear systems
     %# Print cost statistics if no output argument is given
     if (nargout == 0)
-      vmsg = fprintf (1, 'Number of successful steps: %d', vnsteps);
-      vmsg = fprintf (1, 'Number of failed attempts:  %d', vnfailed);
-      vmsg = fprintf (1, 'Number of function calls:   %d', vnfevals);
+      vmsg = fprintf (1, 'Number of successful steps: %d\n', vnsteps);
+      vmsg = fprintf (1, 'Number of failed attempts:  %d\n', vnfailed);
+      vmsg = fprintf (1, 'Number of function calls:   %d\n', vnfevals);
     end
   else
     vhavestats = false;
@@ -627,13 +672,21 @@ end
 %!  assert (vsol.x(:), [0:0.1:2]');
 %!  assert (vsol.y(end,:), fref, 1e-3);
 %!test %# Solve in backward direction starting at t=0
-%! %# vref = [-1.2054034414, 0.9514292694];
+%!  vref = [-1.205364552835178, 0.951542399860817];
 %!  vsol = ode54 (@fpol, [0 -2], [2 0]);
-%! %# assert ([vsol.x(end), vsol.y(end,:)], [-2, fref], 1e-3);
+%!  assert ([vsol.x(end), vsol.y(end,:)], [-2, vref], 1e-3);
 %!test %# Solve in backward direction starting at t=2
-%! %# vref = [-1.2154183302, 0.9433018000];
-%!  vsol = ode54 (@fpol, [2 -2], [0.3233166627 -1.8329746843]);
-%! %# assert ([vsol.x(end), vsol.y(end,:)], [-2, fref], 1e-3);
+%!  vref = [-1.205364552835178, 0.951542399860817];
+%!  vsol = ode54 (@fpol, [2 -2], fref);
+%!  assert ([vsol.x(end), vsol.y(end,:)], [-2, vref], 1e-3);
+%!test %# Solve another anonymous function in backward direction
+%!  vref = [-1, 0.367879437558975];
+%!  vsol = ode54 (@(t,y) y, [0 -1], 1);
+%!  assert ([vsol.x(end), vsol.y(end,:)], vref, 1e-3);
+%!test %# Solve another anonymous function below zero
+%!  vref = [0, 14.77810590694212];
+%!  vsol = ode54 (@(t,y) y, [-2 0], 2);
+%!  assert ([vsol.x(end), vsol.y(end,:)], vref, 1e-3);
 %!test %# AbsTol option
 %!  vopt = odeset ('AbsTol', 1e-5);
 %!  vsol = ode54 (@fpol, [0 2], [2 0], vopt);
